%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This function computes the SQRT-LASSO estimator.
% X design matrix     n x p
% y response variable n x 1
% lambda              penalty parameter 
% gamma               p x 1  (vector of loadings typically: gamma_j = sqrt( En[x_{ij}^2] )
%
% min  sqrt( Qhat(beta) ) + (lambda/n)*\sum_j gamma_j*|beta_j|
%
% Output:   betaSQ  square root LASSO estimator
%           sQR     number of non zero components in betaSQ
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [betaSQ, sSQ] = SqrtLassoIterative(X, y, lambda, gamma,varargin)

[maxIter,verbose,optTol,optTolOBJ,zeroThreshold] = process_options(varargin,'maxIter',10000,'verbose',0,'optTol',1e-6,'optTolOBJ',1e-8,'zeroThreshold',1e-4);

[n p] = size(X);

% Start from the Least Squares solution
MM = eye(p);
for j = 1 : p 
    MM(j,j) = (lambda)*gamma(j);
end
beta = (X'*X + MM)\(X'*y);


% Start the log
    w_old = beta;
    k=1;
    wp = beta;
if verbose==2
    fprintf('%10s %10s %15s %15s %15s %15s\n','iter','shoots','||beta||_1','||beta-beta_old||','f(w)','dual');
end


m = 0;

XX = X'*X/n;
Xy = X'*y/n;

ERROR = y - X*beta;
Qhat = sum( ERROR.^2 )/n;
while m < maxIter
        
    
    beta_old = beta;
    for j = 1:p
               
        

        % Compute the Shoot and Update the variable
        S0 = XX(j,:)*beta - XX(j,j)*beta(j) - Xy(j);
        % ERROR = y - X*beta +  X(:,j)*beta(j);
        if ( abs(beta(j)) > 0 )
            ERROR = ERROR + X(:,j)*beta(j);
            Qhat = sum( ERROR.^2 )/n;
        end
        %%% Note that by C-S
        %%% S0^2 <= Qhat * XX(j,j), so that  Qhat >= S0^2/XX(j,j)  :)

        if ( n^2 < (lambda * gamma(j))^2 / XX(j,j) )
        
            beta(j,1) = 0; 
        
        elseif S0 > (lambda/n) * gamma(j) * sqrt(Qhat)
            %%% Optimal beta(j) < 0
            %beta(j,1) = (lambda - S0)/XX2(j,j); for LASSO
            % for SQRT-LASSO
            beta(j,1) = (  ( lambda * gamma(j) / sqrt( n^2 - (lambda * gamma(j))^2 / XX(j,j)  ) )  *  sqrt( max(Qhat - (S0^2/XX(j,j)),0) )  - S0 )   /   XX(j,j);
        
            ERROR = ERROR - X(:,j)*beta(j);
            
        elseif S0 < - (lambda/n) * gamma(j) * sqrt(Qhat)
            %%% Optimal beta(j) > 0
            % beta(j,1) = (-lambda - S0)/XX2(j,j); for LASSO
            % for SQRT-LASSO
            beta(j,1) = (  -  ( lambda * gamma(j) / sqrt( n^2 - (lambda * gamma(j))^2 / XX(j,j)  ) )  *  sqrt( max( Qhat - (S0^2/XX(j,j)),0) )  - S0 )   /   XX(j,j);
            ERROR = ERROR - X(:,j)*beta(j);
            
        elseif abs(S0) <= (lambda/n) * gamma(j) * sqrt(Qhat)
            beta(j,1) = 0;
        end
        
    end
    
    m = m + 1;
    
    % Update the log
        fobj = sqrt( sum((X*beta-y).^2)/n )  +  (lambda*gamma/n)'*abs(beta);
        
        if ( norm(ERROR) > 0 )
            aaa  = (sqrt(n)*ERROR/norm(ERROR));
            dual = aaa'*y/n - abs(  lambda*gamma/n - abs(X'*aaa/n) )'*abs(beta);
        else
            dual = (lambda*gamma/n)'*abs(beta);
        end
    if verbose==2
        fprintf('%10d %10d %15.2e %15.2e %15.2e %15.2e\n',m,m*p,sum(abs(beta)),sum(abs(beta-w_old)),fobj, dual );
    end
        w_old = beta;
        k=k+1;
        wp(:,k) = beta;
    % Check termination
    if sum(abs(beta-beta_old)) < optTol  
        
        if ( fobj - dual  < optTolOBJ  )
            
            break;
    
        end
        
    
    end
    
    
end

betaSQ = beta;
sSQ = sum( abs(betaSQ) > 0 );
if verbose
fprintf('Number of iterations: %d\nTotal Shoots: %d\n',m,m*p);
fprintf('Number of Nonzero components: %d\n',sSQ);
if ( Qhat > 0 )
    fprintf('Maximal Dual violation: %15.2e (max relative: %5.2e)\n', max( 0, max(  abs(X'*aaa/n) - lambda*gamma/n  ) ), max( 0, max( (abs(X'*aaa/n) - lambda*gamma/n  )./(lambda*gamma/n) ) ) );
else
    
end    

end